"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const ora_1 = tslib_1.__importDefault(require("ora"));
const markdown_provider_1 = require("../core/markdown-provider");
const foam_1 = require("../core/model/foam");
const uri_1 = require("../core/model/uri");
const janitor_1 = require("../core/janitor");
const apply_text_edit_1 = require("../core/janitor/apply-text-edit");
const datastore_1 = require("../core/services/datastore");
const write_file_to_disk_1 = require("../utils/write-file-to-disk");
const utils_1 = require("../utils");
const isNote = (resource) => resource.type === 'note';
class Janitor extends command_1.Command {
    async run() {
        const spinner = ora_1.default('Reading Files').start();
        const { args, flags } = this.parse(Janitor);
        const { workspacePath = './' } = args;
        if (utils_1.isValidDirectory(workspacePath)) {
            const matcher = new datastore_1.Matcher([uri_1.URI.file(workspacePath)], ['**/*'], []);
            const dataStore = new datastore_1.FileDataStore();
            const markdownProvider = new markdown_provider_1.MarkdownResourceProvider(matcher);
            const workspace = (await foam_1.bootstrap(matcher, dataStore, [markdownProvider]))
                .workspace;
            const notes = workspace.list().filter(isNote);
            spinner.succeed();
            spinner.text = `${notes.length} files found`;
            spinner.succeed();
            // exit early if no files found.
            if (notes.length === 0) {
                this.exit();
            }
            spinner.text = 'Generating link definitions';
            const fileWritePromises = notes.map(note => {
                // Get edits
                const heading = janitor_1.generateHeading(note);
                const definitions = janitor_1.generateLinkReferences(note, workspace, !flags['without-extensions']);
                // apply Edits
                let file = note.source.text;
                file = heading ? apply_text_edit_1.applyTextEdit(file, heading) : file;
                file = definitions ? apply_text_edit_1.applyTextEdit(file, definitions) : file;
                if (heading || definitions) {
                    return write_file_to_disk_1.writeFileToDisk(note.uri, file);
                }
                return Promise.resolve(null);
            });
            await Promise.all(fileWritePromises);
            spinner.succeed();
            spinner.succeed('Done!');
        }
        else {
            spinner.fail('Directory does not exist!');
        }
    }
}
exports.default = Janitor;
Janitor.description = 'Updates link references and heading across all the markdown files in the given workspaces';
Janitor.examples = [
    `$ foam-cli janitor path-to-foam-workspace
`,
];
Janitor.flags = {
    'without-extensions': command_1.flags.boolean({
        char: 'w',
        description: 'generate link reference definitions without extensions (for legacy support)',
    }),
    help: command_1.flags.help({ char: 'h' }),
};
Janitor.args = [{ name: 'workspacePath' }];
