"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const github_slugger_1 = tslib_1.__importDefault(require("github-slugger"));
const command_1 = require("@oclif/command");
const ora_1 = tslib_1.__importDefault(require("ora"));
const markdown_provider_1 = require("../core/markdown-provider");
const foam_1 = require("../core/model/foam");
const uri_1 = require("../core/model/uri");
const janitor_1 = require("../core/janitor");
const apply_text_edit_1 = require("../core/janitor/apply-text-edit");
const datastore_1 = require("../core/services/datastore");
const write_file_to_disk_1 = require("../utils/write-file-to-disk");
const rename_file_1 = require("../utils/rename-file");
const utils_1 = require("../utils");
const slugger = new github_slugger_1.default();
const isNote = (resource) => resource.type === 'note';
const getKebabCaseFileName = (fileName) => {
    const kebabCasedFileName = slugger.slug(fileName);
    return kebabCasedFileName === fileName ? null : kebabCasedFileName;
};
// @todo: Refactor 'migrate' and 'janitor' commands and avoid repeatition
class Migrate extends command_1.Command {
    async run() {
        const spinner = ora_1.default('Reading Files').start();
        const { args, flags } = this.parse(Migrate);
        const { workspacePath = './' } = args;
        const matcher = new datastore_1.Matcher([uri_1.URI.file(workspacePath)], ['**/*'], []);
        const dataStore = new datastore_1.FileDataStore();
        const markdownProvider = new markdown_provider_1.MarkdownResourceProvider(matcher);
        if (utils_1.isValidDirectory(workspacePath)) {
            let workspace = (await foam_1.bootstrap(matcher, dataStore, [markdownProvider])).workspace;
            let notes = workspace.list().filter(isNote);
            spinner.succeed();
            spinner.text = `${notes.length} files found`;
            spinner.succeed();
            // exit early if no files found.
            if (notes.length === 0) {
                this.exit();
            }
            // Kebab case file names
            const fileRename = notes.map(note => {
                if (note.title != null) {
                    const kebabCasedFileName = getKebabCaseFileName(note.title);
                    if (kebabCasedFileName) {
                        return rename_file_1.renameFile(note.uri, kebabCasedFileName);
                    }
                }
                return Promise.resolve(null);
            });
            await Promise.all(fileRename);
            spinner.text = 'Renaming files';
            // Reinitialize the graph after renaming files
            workspace = (await foam_1.bootstrap(matcher, dataStore, [markdownProvider])).workspace;
            notes = workspace.list().filter(isNote);
            spinner.succeed();
            spinner.text = 'Generating link definitions';
            const fileWritePromises = await Promise.all(notes.map(note => {
                // Get edits
                const heading = janitor_1.generateHeading(note);
                const definitions = janitor_1.generateLinkReferences(note, workspace, !flags['without-extensions']);
                // apply Edits
                let file = note.source.text;
                file = heading ? apply_text_edit_1.applyTextEdit(file, heading) : file;
                file = definitions ? apply_text_edit_1.applyTextEdit(file, definitions) : file;
                if (heading || definitions) {
                    return write_file_to_disk_1.writeFileToDisk(note.uri, file);
                }
                return Promise.resolve(null);
            }));
            await Promise.all(fileWritePromises);
            spinner.succeed();
            spinner.succeed('Done!');
        }
        else {
            spinner.fail('Directory does not exist!');
        }
    }
}
exports.default = Migrate;
Migrate.description = 'Updates file names, link references and heading across all the markdown files in the given workspaces';
Migrate.examples = [
    `$ foam-cli migrate path-to-foam-workspace
Successfully generated link references and heading!
`,
];
Migrate.flags = {
    'without-extensions': command_1.flags.boolean({
        char: 'w',
        description: 'generate link reference definitions without extensions (for legacy support)',
    }),
    help: command_1.flags.help({ char: 'h' }),
};
Migrate.args = [{ name: 'workspacePath' }];
